@@ -20,6 +20,7 @@
 Database models of the Dashboard application
 """
 
+import ast
 import datetime
 import errno
 import gzip
@@ -51,8 +52,8 @@ from django.dispatch import receiver
 from django.template import Template, Context
 from django.template.defaultfilters import filesizeformat, slugify
 from django.template.loader import render_to_string
-from django.utils.translation import ugettext as _
-from django.utils.translation import ungettext
+from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import ungettext_lazy
 from django.db.utils import DatabaseError
 from django_restricted_resource.models import RestrictedResource
 from linaro_dashboard_bundle.io import DocumentIO
@@ -62,13 +63,8 @@ from dashboard_app.managers import BundleManager, TestRunDenormalizationManager
 from dashboard_app.signals import bundle_was_deserialized
 
 
-# Fix some django issues we ran into
-from dashboard_app.patches import patch
-patch()
-
-
 def _help_max_length(max_length):
-    return ungettext(
+    return ungettext_lazy(
         u"Maximum length: {0} character",
         u"Maximum length: {0} characters",
         max_length).format(max_length)
@@ -212,7 +208,7 @@ class BundleStream(RestrictedResource):
         unique=True,
     )
 
-    is_anonymous = models.BooleanField()
+    is_anonymous = models.BooleanField(default=False)
 
     def __unicode__(self):
         return self.pathname
@@ -483,7 +479,8 @@ class Bundle(models.Model):
         verbose_name=_(u"Is deserialized"),
         help_text=_(u"Set when document has been analyzed and loaded"
                     " into the database"),
-        editable=False)
+        editable=False,
+        default=False)
 
     _raw_content = models.FileField(
         verbose_name=_(u"Content"),
@@ -629,6 +626,8 @@ class Bundle(models.Model):
             try:
                 fmt, doc = DocumentIO.load(self.content)
                 return fmt
+            except Exception:
+                return "unknown"
             finally:
                 self.content.close()
 
@@ -718,10 +717,6 @@ class Test(models.Model):
     def __unicode__(self):
         return self.name or self.test_id
 
-    @models.permalink
-    def get_absolute_url(self):
-        return self.test_id
-
     def count_results_without_test_case(self):
         return TestResult.objects.filter(
             test_run__test=self,
@@ -768,10 +763,6 @@ class TestCase(models.Model):
     def __unicode__(self):
         return self.name or self.test_case_id
 
-    @models.permalink
-    def get_absolute_url(self):
-        return ("dashboard_app.test_case.details", [self.test.test_id, self.test_case_id])
-
     def count_failures(self):
         return self.test_results.filter(result=TestResult.RESULT_FAIL).count()
 
@@ -982,8 +973,8 @@ class TestRun(models.Model):
                     "internet time servers.<br/>"
                     "This field allows us to track tests results that "
                     "<em>certainly</em> have correct time if we ever end up "
-                    "with lots of tests results from 1972")
-    )
+                    "with lots of tests results from 1972"),
+        default=False)
 
     microseconds = models.BigIntegerField(
         blank=True,
@@ -1134,6 +1125,21 @@ class TestRun(models.Model):
                 return one_attributes.value
         return "Target Device"
 
+    def get_test_params(self):
+        """When test_params are available return it as a dict after converting
+        the dict to contain normal strings without unicode notation. If there
+        are no test parameters, then we return None.
+        """
+        for src in self.sources.all():
+            if src.test_params:
+                test_struct = ast.literal_eval(src.test_params)
+                if type(test_struct) == dict:
+                    test_params = {}
+                    for k, v in test_struct.items():
+                        test_params[str(k)] = str(v)
+                    return test_params
+        return None
+
 
 class TestRunDenormalization(models.Model):
     """
@@ -1629,15 +1635,7 @@ class TestRunFilter(models.Model):
     #         or filter.test_case in select test_case from bundle.test_runs.test_results.test_cases)
 
     @classmethod
-    def matches_against_bundle(self, bundle):
-        cursor = connection.cursor()
-        try:
-            cursor.execute("select id from dashboard_app_testrunfilterattribute where (name,value) not in ("
-                           "select name, value from dashboard_app_namedattribute where content_type_id = ("
-                           "select django_content_type.id from django_content_type "
-                           "where app_label = 'dashboard_app' and model='testrun'));")
-        except DatabaseError:
-            return []
+    def matches_against_bundle(cls, bundle):
         from dashboard_app.filters import FilterMatch
         bundle_filters = bundle.bundle_stream.testrunfilter_set.all()
         attribute_filters = bundle_filters.extra(
@@ -1793,13 +1791,29 @@ def send_image_report_notifications(sender, bundle):
                                     if denorm.count_pass < chart.target_goal:
                                         matches.append(test_run)
 
-                        else:
+                        elif chart.chart_type == "measurement":
                             for test_result in filter_match.specific_results:
                                 if test_result.test_case in chart_tests:
                                     if test_result.measurement <\
                                             chart.target_goal:
                                         matches.append(test_result)
 
+                        elif chart.chart_type == "attributes":
+                            for test_run in filter_match.test_runs:
+                                if test_run.test in chart_tests:
+                                    for attr in chart_test.attributes:
+                                        try:
+                                            value = float(
+                                                test_run.attributes.get(
+                                                    name=attr).value)
+                                        except ValueError:
+                                            # Do not user attributes which are
+                                            # not numbers.
+                                            continue
+                                        if value < chart.target_goal:
+                                            if test_run not in matches:
+                                                matches.append(test_run)
+
                 for chart_user in chart.imagechartuser_set.all():
                     if matches:
                         title = "LAVA image report test failure notification: %s" % chart.name
@@ -1901,7 +1915,8 @@ class ImageReport(models.Model):
 
 # Chart types
 CHART_TYPES = ((r'pass/fail', 'Pass/Fail'),
-               (r'measurement', 'Measurement'))
+               (r'measurement', 'Measurement'),
+               (r'attributes', 'Attributes'))
 # Chart representation
 REPRESENTATION_TYPES = ((r'lines', 'Lines'),
                         (r'bars', 'Bars'))
@@ -1975,10 +1990,14 @@ class ImageReportChart(models.Model):
                 self.get_chart_test_data(user, image_chart_filter, filter_data,
                                          chart_data)
 
-            else:
+            elif self.chart_type == "measurement":
                 self.get_chart_test_case_data(user, image_chart_filter,
                                               filter_data, chart_data)
 
+            elif self.chart_type == "attributes":
+                self.get_chart_attributes_data(user, image_chart_filter,
+                                               filter_data, chart_data)
+
         return chart_data
 
     def get_basic_chart_data(self):
@@ -2006,11 +2025,41 @@ class ImageReportChart(models.Model):
             chart_data["start_date"] = chart_user.start_date
             chart_data["is_legend_visible"] = chart_user.is_legend_visible
             chart_data["has_subscription"] = chart_user.has_subscription
+            chart_data["toggle_percentage"] = chart_user.toggle_percentage
 
         except ImageChartUser.DoesNotExist:
             # Leave an empty dict.
             pass
 
+        chart_data["hidden_tests"] = []
+        if self.chart_type == "pass/fail":
+
+            chart_test_users = ImageChartTestUser.objects.filter(
+                image_chart_test__image_chart_filter__image_chart=self)
+
+            for chart_test_user in chart_test_users:
+                if not chart_test_user.is_visible:
+                    chart_data["hidden_tests"].append(
+                        chart_test_user.image_chart_test.id)
+        elif self.chart_type == "measurement":
+            chart_test_case_users = ImageChartTestCaseUser.objects.filter(
+                image_chart_test_case__image_chart_filter__image_chart=self)
+            for chart_test_case_user in chart_test_case_users:
+                if not chart_test_case_user.is_visible:
+                    chart_data["hidden_tests"].append(
+                        chart_test_case_user.image_chart_test_case.id)
+        elif self.chart_type == "attributes":
+            chart_users = ImageChartTestAttributeUser.objects.filter(
+                image_chart_test_attribute__image_chart_test__image_chart_filter__image_chart=self)
+
+            for chart_user in chart_users:
+                if not chart_user.is_visible:
+                    test_id = chart_user.image_chart_test_attribute.\
+                        image_chart_test.id
+                    chart_data["hidden_tests"].append(
+                        "%s-%s" % (test_id,
+                                   chart_user.image_chart_test_attribute.name))
+
         return chart_data
 
     def get_chart_test_data(self, user, image_chart_filter, filter_data,
@@ -2055,6 +2104,7 @@ class ImageReportChart(models.Model):
                 test_id = test_run.test.test_id
 
                 # Find corresponding chart_test object.
+                chart_test = None
                 for ch_test in selected_chart_tests:
                     if ch_test.test == test_run.test:
                         chart_test = ch_test
@@ -2064,7 +2114,9 @@ class ImageReportChart(models.Model):
                     metadata[test_id] = {}
 
                 alias = None
+                chart_test_id = None
                 if chart_test:
+                    chart_test_id = chart_test.id
                     # Metadata delta content. Contains attribute names as keys
                     # and value is tuple with old and new value.
                     # If specific attribute's value didn't change since the
@@ -2097,23 +2149,43 @@ class ImageReportChart(models.Model):
                     comments__isnull=True).count() != 0
 
                 test_filter_id = "%s-%s" % (test_id, image_chart_filter.id)
-                chart_item = {
-                    "filter_rep": image_chart_filter.representation,
-                    "test_filter_id": test_filter_id,
-                    "link": test_run.get_absolute_url(),
-                    "alias": alias,
-                    "number": str(match.tag),
-                    "date": str(test_run.bundle.uploaded_on),
-                    "pass": denorm.count_fail == 0,
-                    "passes": denorm.count_pass,
-                    "total": denorm.count_pass + denorm.count_fail,
-                    "test_run_uuid": test_run.analyzer_assigned_uuid,
-                    "bug_links": bug_links,
-                    "metadata_content": metadata_content,
-                    "comments": has_comments,
-                }
 
-                chart_data["test_data"].append(chart_item)
+                # Find already existing chart item (this happens if we're
+                # dealing with parametrized tests) and add the values instead
+                # of creating new chart item.
+                found = False
+                for chart_item in chart_data["test_data"]:
+                    if chart_item["test_filter_id"] == test_filter_id and \
+                            chart_item["number"] == str(match.tag):
+                        chart_item["passes"] += denorm.count_pass
+                        chart_item["skip"] += denorm.count_skip
+                        chart_item["total"] += denorm.count_all()
+                        chart_item["link"] = test_run.bundle.get_absolute_url()
+                        chart_item["pass"] &= denorm.count_fail == 0
+                        found = True
+
+                # If no existing chart item was found, create a new one.
+                if not found:
+                    chart_item = {
+                        "filter_rep": image_chart_filter.representation,
+                        "test_filter_id": test_filter_id,
+                        "chart_test_id": chart_test_id,
+                        "link": test_run.get_absolute_url(),
+                        "bundle_link": test_run.bundle.get_absolute_url(),
+                        "alias": alias,
+                        "number": str(match.tag),
+                        "date": str(test_run.bundle.uploaded_on),
+                        "pass": denorm.count_fail == 0,
+                        "passes": denorm.count_pass,
+                        "skip": denorm.count_skip,
+                        "total": denorm.count_all(),
+                        "test_run_uuid": test_run.analyzer_assigned_uuid,
+                        "bug_links": bug_links,
+                        "metadata_content": metadata_content,
+                        "comments": has_comments,
+                    }
+
+                    chart_data["test_data"].append(chart_item)
 
     def get_chart_test_case_data(self, user, image_chart_filter, filter_data,
                                  chart_data):
@@ -2157,14 +2229,17 @@ class ImageReportChart(models.Model):
                 if test_case_id not in metadata.keys():
                     metadata[test_case_id] = {}
 
-                # Find corresponding chart_test object.
+                # Find corresponding chart_test_case object.
+                chart_test_case = None
                 for ch_test_case in selected_chart_test_cases:
                     if ch_test_case.test_case == test_result.test_case:
                         chart_test_case = ch_test_case
                         break
 
                 alias = None
+                chart_test_id = None
                 if chart_test_case:
+                    chart_test_id = chart_test_case.id
                     # Metadata delta content. Contains attribute names as
                     # keys and value is tuple with old and new value.
                     # If specific attribute's value didn't change since the
@@ -2201,6 +2276,7 @@ class ImageReportChart(models.Model):
                     "filter_rep": image_chart_filter.representation,
                     "alias": alias,
                     "test_filter_id": test_filter_id,
+                    "chart_test_id": chart_test_id,
                     "units": test_result.units,
                     "measurement": test_result.measurement,
                     "link": test_result.get_absolute_url(),
@@ -2214,6 +2290,88 @@ class ImageReportChart(models.Model):
                 }
                 chart_data["test_data"].append(chart_item)
 
+    def get_chart_attributes_data(self, user, image_chart_filter, filter_data,
+                                  chart_data):
+        from dashboard_app.filters import evaluate_filter
+
+        # Prepare to filter the tests and test cases for the
+        # evaluate_filter call.
+        tests = []
+
+        selected_chart_tests = image_chart_filter.imagecharttest_set.all().prefetch_related('imagecharttestattribute_set')
+
+        # Leave chart_data empty if there are no tests available.
+        if not selected_chart_tests:
+            return
+
+        for chart_test in selected_chart_tests:
+            tests.append({
+                'test': chart_test.test,
+                'test_cases': [],
+            })
+
+        filter_data['tests'] = tests
+
+        matches = list(evaluate_filter(user, filter_data,
+                                       prefetch_related=['bug_links'])[:50])
+        matches.reverse()
+
+        for match in matches:
+            for test_run in match.test_runs:
+
+                denorm = test_run.denormalization
+
+                bug_links = sorted(
+                    [b.bug_link for b in test_run.bug_links.all()])
+
+                test_id = test_run.test.test_id
+
+                # Find corresponding chart_test object.
+                chart_test = None
+                for ch_test in selected_chart_tests:
+                    if ch_test.test == test_run.test:
+                        chart_test = ch_test
+                        break
+
+                alias = None
+                chart_test_id = None
+                if chart_test:
+                    chart_test_id = chart_test.id
+                    for attr in chart_test.attributes:
+                        try:
+                            value = float(
+                                test_run.attributes.get(name=attr).value)
+                        except (NamedAttribute.DoesNotExist, ValueError):
+                            # Skip this attribute.
+                            continue
+
+                        if chart_test.name:
+                            test_name = chart_test.name
+                        else:
+                            test_name = test_id
+
+                        alias = "%s: %s" % (test_name,
+                                            attr.replace(" ", ""))
+
+                        test_filter_id = "%s-%s-%s" % (test_id,
+                                                       image_chart_filter.id,
+                                                       attr.replace(" ", ""))
+                        chart_item = {
+                            "filter_rep": image_chart_filter.representation,
+                            "test_filter_id": test_filter_id,
+                            "chart_test_id": "%s-%s" % (chart_test_id, attr),
+                            "link": test_run.get_absolute_url(),
+                            "alias": alias,
+                            "number": str(match.tag),
+                            "date": str(test_run.bundle.uploaded_on),
+                            "pass": denorm.count_fail == 0,
+                            "attr_value": value,
+                            "test_run_uuid": test_run.analyzer_assigned_uuid,
+                            "bug_links": bug_links,
+                        }
+
+                        chart_data["test_data"].append(chart_item)
+
 
 class ImageChartFilter(models.Model):
 
@@ -2306,6 +2464,8 @@ class ImageChartTest(models.Model):
         filter_data = self.image_chart_filter.filter.as_data()
         filter_data['tests'] = tests
         matches = list(evaluate_filter(user, filter_data)[:1])
+        if not matches or not matches[0].test_runs:
+            return list()
         test_run_id = matches[0].test_runs[0].id
 
         result = NamedAttribute.objects.all()
@@ -2375,6 +2535,11 @@ class ImageChartTestCase(models.Model):
         filter_data = self.image_chart_filter.filter.as_data()
         filter_data['tests'] = tests
         matches = list(evaluate_filter(user, filter_data)[:1])
+
+        # If no matches are found, return empty list.
+        if len(matches) == 0:
+            return list()
+
         test_run_id = matches[0].test_runs[0].id
 
         result = NamedAttribute.objects.all()
@@ -2423,3 +2588,76 @@ class ImageChartUser(models.Model):
     has_subscription = models.BooleanField(
         default=False,
         verbose_name='Subscribed to target goal')
+
+    toggle_percentage = models.BooleanField(
+        default=False,
+        verbose_name='Toggle percentage')
+
+
+class ImageChartTestUser(models.Model):
+    """
+    Stores information which tests user has hidden in the image chart.
+    """
+
+    class Meta:
+        unique_together = ("image_chart_test", "user")
+
+    image_chart_test = models.ForeignKey(
+        ImageChartTest,
+        null=False,
+        on_delete=models.CASCADE)
+
+    user = models.ForeignKey(
+        User,
+        null=False,
+        on_delete=models.CASCADE)
+
+    is_visible = models.BooleanField(
+        default=True,
+        verbose_name='Visible')
+
+
+class ImageChartTestCaseUser(models.Model):
+    """
+    Stores information which test cases user has hidden in the image chart.
+    """
+
+    class Meta:
+        unique_together = ("image_chart_test_case", "user")
+
+    image_chart_test_case = models.ForeignKey(
+        ImageChartTestCase,
+        null=False,
+        on_delete=models.CASCADE)
+
+    user = models.ForeignKey(
+        User,
+        null=False,
+        on_delete=models.CASCADE)
+
+    is_visible = models.BooleanField(
+        default=True,
+        verbose_name='Visible')
+
+
+class ImageChartTestAttributeUser(models.Model):
+    """
+    Stores information which attributes are hidden in the image chart.
+    """
+
+    class Meta:
+        unique_together = ("image_chart_test_attribute", "user")
+
+    image_chart_test_attribute = models.ForeignKey(
+        ImageChartTestAttribute,
+        null=False,
+        on_delete=models.CASCADE)
+
+    user = models.ForeignKey(
+        User,
+        null=False,
+        on_delete=models.CASCADE)
+
+    is_visible = models.BooleanField(
+        default=True,
+        verbose_name='Visible')
