@@ -26,40 +26,12 @@ $(document).ready(function () {
         this.chart_id = chart_id;
         this.chart_data = chart_data;
         this.legend_items = [];
+        this.update_alias();
     }
 
     ImageChart.prototype.BUILD_NUMBER_ERROR =
         "End build number must be greater then the start build number.";
 
-    ImageChart.prototype.setup_clickable = function() {
-
-        var chart = this;
-        $("#legend_container_" + this.chart_id + " table:first-child tbody tr").each(function(index) {
-
-            $(this).click(function() {
-
-                if (chart.legend_items[index].show == true) {
-                    chart.legend_items[index].show = false;
-                } else {
-                    chart.legend_items[index].show = true;
-                }
-
-                chart.update_plot();
-            });
-        });
-    }
-
-    ImageChart.prototype.setup_legend_css = function() {
-
-        for (var i in this.legend_items) {
-            if (this.legend_items[i].show == true) {
-                $("#" + this.legend_items[i].dom_id).parent().css("color", "#545454");
-            } else {
-                $("#" + this.legend_items[i].dom_id).parent().css("color", "#999");
-            }
-        }
-    }
-
     ImageChart.prototype.add_chart = function() {
 
         if (this.chart_data.test_data) {
@@ -70,15 +42,15 @@ $(document).ready(function () {
             $("#chart_container_" + this.chart_id).append(
                 '<div class="headline-container" id="headline_container_' +
                     this.chart_id + '"></div>');
+            // Add data table link.
+            $("#chart_container_" + this.chart_id).append(
+                '<div class="table-link-container"' +
+                    'id="table_link_container_' + this.chart_id + '"></div>');
             // Add filter links used.
             $("#chart_container_" + this.chart_id).append(
                 '<div class="filter-links-container"' +
                     'id="filter_links_container_' + this.chart_id +
                     '"></div>');
-            // Add data table link.
-            $("#filter_links_container_" + this.chart_id).append(
-                '<span class="table-link-container"' +
-                    'id="table_link_container_' + this.chart_id + '"></span>');
             // Add dates/build numbers container.
             $("#chart_container_" + this.chart_id).append(
                 '<div class="dates-container" id="dates_container_' +
@@ -98,10 +70,10 @@ $(document).ready(function () {
 
             // Add headline and description.
             this.update_headline();
-            // Add dates/build numbers.
-            this.update_dates();
             // Add filter links.
             this.update_filter_links();
+            // Add dates/build numbers.
+            this.update_dates();
             // Add data tables.
             this.update_data_tables();
             // Generate chart.
@@ -113,6 +85,20 @@ $(document).ready(function () {
         }
     }
 
+
+    ImageChart.prototype.update_alias = function() {
+        if (Object.keys(this.chart_data.filters).length == 1) {
+            for (iter in this.chart_data.test_data) {
+                test_data = this.chart_data.test_data[iter];
+                if (test_data["alias"].split(": ").length > 1) {
+                    test_data["alias"] =
+                        test_data["alias"].split(": ").slice(1).join(": ");
+                }
+            }
+        }
+    }
+
+
     ImageChart.prototype.setup_print_menu = function() {
         chart_id = this.chart_id;
         $("#print_menu_" + chart_id).menu();
@@ -152,7 +138,6 @@ $(document).ready(function () {
             });
     }
 
-
     ImageChart.prototype.update_headline = function() {
         $("#headline_container_" + this.chart_id).append(
             '<span class="chart-headline">' + this.chart_data["name"] +
@@ -164,21 +149,68 @@ $(document).ready(function () {
     ImageChart.prototype.update_filter_links = function() {
 
         $("#filter_links_container_" + this.chart_id).append(
-            '<span style="margin-left: 30px;">Filters used:&nbsp;&nbsp;' +
-                '</span>');
+            '<span style="margin-left: 30px;"><a id="filter_link_' +
+                this.chart_id + '"' +
+                ' href="javascript:void(0)">Filters used</a></span>');
+
+        // Add dialog.
+        $("#main_container").append('<div id="filter_links_dialog_' +
+                                    this.chart_id + '"></div>');
+
+        // Init dialog.
+        $('#filter_links_dialog_' + this.chart_id).dialog({
+            autoOpen: false,
+            title: 'Filters used',
+            draggable: false,
+            height: 480,
+            width: 450,
+            modal: true,
+            resizable: false,
+        });
+
+        var chart = this;
+        $('#filter_link_' + this.chart_id).click(function() {
+            $('#filter_links_dialog_' + chart.chart_id).dialog("open");
+        });
+
+
         filter_links = [];
+        height = 80;
         for (filter_id in this.chart_data.filters) {
-            filter_links.push('<a href="' +
-                              this.chart_data.filters[filter_id]["link"] +
+            filter_links.push('<a target="_blank" href="' +
+                              this.chart_data.filters[filter_id]["link"].replace(/\\/g , "") +
                               '">~' +
                               this.chart_data.filters[filter_id]["owner"] +
                               '/' +
                               this.chart_data.filters[filter_id]["name"] +
                               '</a>');
+            height += 30;
         }
-        filter_html = filter_links.join(", ");
+        filter_html = filter_links.join("<br>");
+
+        $("#filter_links_dialog_" + this.chart_id).append('<div>' + filter_html + '</div>');
+        $("#filter_links_dialog_" + this.chart_id).dialog("option",
+                                                          "height", height );
+
+        // Add percentages if chart type is pass/fail.
+        if (this.chart_data["chart_type"] == "pass/fail") {
+            $("#filter_links_container_" + this.chart_id).append(
+                '<span class="toggle-percentage"><label for="is_percentage_' +
+                    this.chart_id +
+                    '">Toggle percentage</label></span>');
+            $("#filter_links_container_" + this.chart_id).append(
+                '<span class="toggle-checkbox"><input type="checkbox" ' +
+                    'id="is_percentage_' + this.chart_id + '" /></span>');
+        }
+
+        // Add legend toggle checkbox.
         $("#filter_links_container_" + this.chart_id).append(
-            '<span>' + filter_html + '</span>');
+            '<span class="toggle-legend"><label for="is_legend_visible_' +
+                this.chart_id + '">Toggle legend</label></span>');
+        $("#filter_links_container_" + this.chart_id).append(
+            '<span class="toggle-checkbox"><input type="checkbox" ' +
+                'id="is_legend_visible_' + this.chart_id +
+                '" checked="checked"/></span>');
 
         $("#filter_links_container_" + this.chart_id).append(
             '<span class="chart-save-img">' +
@@ -215,24 +247,6 @@ $(document).ready(function () {
         $("#dates_container_" + this.chart_id).append(
             '<span><select id="end_date_' + this.chart_id + '"></select>' +
                 '</span>');
-
-        // Add percentages if chart type is pass/fail.
-        if (this.chart_data["chart_type"] == "pass/fail") {
-            $("#dates_container_" + this.chart_id).append(
-                '<span>&nbsp;&nbsp;&nbsp;&nbsp;<label for="is_percentage_' +
-                    this.chart_id +
-                    '">Toggle percentage:</label>&nbsp;&nbsp;</span>');
-            $("#dates_container_" + this.chart_id).append(
-                '<span><input type="checkbox" id="is_percentage_' +
-                    this.chart_id + '" /></span>');
-        }
-
-        $("#dates_container_" + this.chart_id).append(
-            '<span>&nbsp;&nbsp;&nbsp;&nbsp;<label for="is_legend_visible_' +
-                this.chart_id + '">Toggle legend:</label>&nbsp;&nbsp;</span>');
-        $("#dates_container_" + this.chart_id).append(
-            '<span><input type="checkbox" id="is_legend_visible_' +
-                this.chart_id + '" checked="checked"/></span>');
         $("#dates_container_" + this.chart_id).append(
             '<span style="float: right;"><input id="has_subscription_' +
                 this.chart_id +
@@ -253,10 +267,10 @@ $(document).ready(function () {
         // Init dialog.
         $('#data_table_dialog_' + this.chart_id).dialog({
             autoOpen: false,
-            title: 'View data table',
+            title: 'Results table',
             draggable: false,
             height: 280,
-            width: 950,
+            width: 1050,
             modal: true,
             resizable: false,
             open: function (event, ui) {
@@ -273,7 +287,7 @@ $(document).ready(function () {
         $("#data_table_dialog_" + this.chart_id).append(
             '<table id="outer-table"><tr><td>' +
                 '<table id="test-run-names_' + this.chart_id +
-                '" class="inner-table"><thead>' +
+                '" class="inner-table-names"><thead>' +
                 '<tr><th>Build Number</th></tr>' +
                 '</thead>' +
                 '<tbody></tbody></table></td>' +
@@ -290,7 +304,7 @@ $(document).ready(function () {
         $("#table_link_container_" + this.chart_id).append(
             '<a id="data_table_link_' +
                 this.chart_id +
-                '" href="javascript:void(0)">View data table</a>');
+                '" href="javascript:void(0)">Results table</a>');
 
         var chart = this;
         $("#data_table_link_" + this.chart_id).click(function(){
@@ -319,11 +333,8 @@ $(document).ready(function () {
 
                 rows.push(test_data["test_filter_id"]);
 
-                test_name = test_data["alias"];
-                if (test_name.length > 10) {
-                    test_name = test_name.substring(0,10) + "...";
-                }
-                table_rows += "<tr><td title='" + test_data["alias"] +
+                test_name = test_data["alias"].replace(/\\/g , "");
+                table_rows += "<tr><td title='" + test_data["alias"].replace(/\\/g , "") +
                     "'>" + test_name + "</td></tr>";
             }
         }
@@ -347,8 +358,11 @@ $(document).ready(function () {
 
             table[number][test_data["test_filter_id"]].push({
                 "passes": test_data["passes"],
+                "pass": test_data["pass"],
+                "skip": test_data["skip"],
                 "total": test_data["total"],
                 "measurement": test_data["measurement"],
+                "attr_value": test_data["attr_value"],
                 "link": test_data["link"],
                 "test_run_uuid": test_data["test_run_uuid"],
                 "bug_links": test_data["bug_links"]
@@ -406,10 +420,10 @@ $(document).ready(function () {
 
                     // Calculate td class.
                     cell = table[number][filter_id][cnt];
-                    if (cell["passes"] < cell["total"]) {
-                        cls = "fail";
-                    } else {
+                    if (cell["pass"]) {
                         cls = "pass";
+                    } else {
+                        cls = "fail";
                     }
                     uuid = cell["test_run_uuid"];
                     relative_index_str = "";
@@ -426,9 +440,12 @@ $(document).ready(function () {
                         table_body += '<a target="_blank" href="' +
                             cell["link"] + '">' + cell["passes"] + '/' +
                             cell["total"] + '</a>';
-                    } else {
+                    } else if (this.chart_data["chart_type"] == "measurement") {
                         table_body += '<a target="_blank" href="' +
                             cell["link"] + '">' + cell["measurement"] + '</a>';
+                    } else if (this.chart_data["chart_type"] == "attributes") {
+                        table_body += '<a target="_blank" href="' +
+                            cell["link"] + '">' + cell["attr_value"] + '</a>';
                     }
 
                     table_body += '<span class="bug-link-container">' +
@@ -548,6 +565,11 @@ $(document).ready(function () {
             chart.update_settings();
         });
 
+        $("#is_percentage_"+this.chart_id).change(function() {
+            chart.update_plot();
+            chart.update_settings();
+        });
+
         if (this.chart_data["chart_type"] == "pass/fail") {
             $("#is_percentage_"+this.chart_id).change(function() {
                 chart.update_plot();
@@ -572,12 +594,25 @@ $(document).ready(function () {
                 $("#is_legend_visible_" + this.chart_id).prop("checked",
                                                               false);
             }
+            if (this.chart_data.user.toggle_percentage == true) {
+                $("#is_percentage_" + this.chart_id).prop("checked",
+                                                              true);
+            }
 
             this.set_subscription_link(this.chart_data.user.has_subscription);
         }
     }
 
-    ImageChart.prototype.update_settings = function() {
+    ImageChart.prototype.update_settings = function(visible_chart_test_id,
+                                                    attr_name) {
+
+        if (typeof(visible_chart_test_id) === 'undefined') {
+            visible_chart_test_id = 0;
+        }
+
+        if (typeof(attr_name) === 'undefined') {
+            attr_name = "";
+        }
 
         url = "/dashboard/image-charts/" + this.chart_data["report_name"] +
             "/" + this.chart_id + "/+settings-update";
@@ -592,6 +627,9 @@ $(document).ready(function () {
                 start_date: $("#start_date_" + this.chart_id).val(),
                 is_legend_visible: $("#is_legend_visible_" + this.chart_id).prop("checked"),
                 has_subscription: $("#has_subscription_" + this.chart_id).val(),
+                toggle_percentage: $("#is_percentage_" + this.chart_id).prop("checked"),
+                visible_chart_test_id: visible_chart_test_id,
+                visible_attribute_name: attr_name,
             },
             success: function (data) {
                 chart.set_subscription_link(data[0].fields.has_subscription);
@@ -601,7 +639,7 @@ $(document).ready(function () {
 
     ImageChart.prototype.set_subscription_link = function(subscribed) {
 
-        if (this.chart_data["target_goal"]) {
+        if (this.chart_data["target_goal"] != null) {
             if (subscribed) {
                 $("#has_subscription_"+this.chart_id).val(true);
                 $("#has_subscription_link_"+this.chart_id).html(
@@ -651,11 +689,14 @@ $(document).ready(function () {
 	    test_filter_id = row["test_filter_id"];
             if (!(test_filter_id in plot_data)) {
                 plot_data[test_filter_id] = {
-                    "alias": row["alias"],
+                    "alias": row["alias"].replace(/\\/g , ""),
                     "representation": row["filter_rep"],
                     "data": [],
                     "meta": [],
-                    "labels": []
+                    "labels": [],
+                    "max": - Number.MAX_VALUE,
+                    "min": Number.MAX_VALUE,
+                    "sum": 0
                 };
             }
         }
@@ -665,6 +706,11 @@ $(document).ready(function () {
         // Store all build numbers
         build_numbers = [];
 
+        // Grid maximum and minimum values for y axis.
+        var y_max = - Number.MAX_VALUE;
+        var y_max_pass = - Number.MAX_VALUE;
+        var y_min = Number.MAX_VALUE;
+
         for (iter in this.chart_data.test_data) {
 
 	    row = this.chart_data.test_data[iter];
@@ -685,21 +731,35 @@ $(document).ready(function () {
                 if (this.chart_data["chart_type"] == "pass/fail") {
                     if ($("#is_percentage_" + this.chart_id).prop("checked") == true) {
                         value = parseFloat(row["passes"]/row["total"]).toFixed(4) * 100;
+                        // Happens when total is 0.
+                        if (isNaN(value)) {
+                            value = 0;
+                        }
                         tooltip = "Pass rate: " + value + "%";
                     } else {
                         value = row["passes"];
                         tooltip = "Pass: " + value + ", Total: " +
-                            row["total"];
+                            row["total"] + ", Skip: " + row["skip"];
                     }
 
-                } else {
+                } else if (this.chart_data["chart_type"] == "measurement") {
                     value = row["measurement"];
                     tooltip = "Value: " + value;
+
+                } else if (this.chart_data["chart_type"] == "attributes") {
+                    value = row["attr_value"];
+                    tooltip = "Value: " + value;
                 }
 
+
                 tooltip += "<br>";
                 label = "";
 
+                // Calculate maximum passes.
+                if (row["passes"] > y_max_pass) {
+                    y_max_pass = row["passes"];
+                }
+
                 // Support metadata content with image and tooltip text.
                 if (!$.isEmptyObject(row["metadata_content"])) {
                     label = "/static/dashboard_app/images/metadata.png";
@@ -728,6 +788,10 @@ $(document).ready(function () {
                     "tooltip": tooltip,
                 };
 
+                // Add chart test/test case id.
+                plot_data[test_filter_id]["chart_test_id"] = row["chart_test_id"];
+
+                // Add the data.
                 if (this.chart_data.has_build_numbers) {
                     insert_data_item(build_number, [build_number, value],
                                      plot_data[test_filter_id]["data"]);
@@ -748,6 +812,15 @@ $(document).ready(function () {
                     plot_data[test_filter_id]["meta"][data_item.join("_")] =
                         meta_item;
                 }
+
+                // Calculate max, min and avg for statistics.
+                if (value < plot_data[test_filter_id]["min"]) {
+                    plot_data[test_filter_id]["min"] = value;
+                }
+                if (value > plot_data[test_filter_id]["max"]) {
+                    plot_data[test_filter_id]["max"] = value;
+                }
+                plot_data[test_filter_id]["sum"] = plot_data[test_filter_id]["sum"] + value;
             }
         }
 
@@ -789,19 +862,26 @@ $(document).ready(function () {
             }
         }
 
-        // Grid maximum and minimum values for y axis.
-        var y_max = - Number.MAX_VALUE;
-        var y_min = Number.MAX_VALUE;
-
         // Pack data in series for plot display.
         for (var i in sorted_filter_ids) {
             test_filter_id = sorted_filter_ids[i];
-
             if (this.legend_items.length != sorted_filter_ids.length) {
+
+                // Load hidden tests data.
+                var show = true;
+                if (this.chart_data.user) { // Is authenticated.
+                    if (this.chart_data.user["hidden_tests"].indexOf(plot_data[test_filter_id]["chart_test_id"]) != -1 ) {
+                        show = false;
+                    }
+                }
+
                 this.legend_items.push({
-                    filter_id: test_filter_id,
-                    dom_id: "legend_" + test_filter_id,
-                    show: true,
+                    chart_test_id: plot_data[test_filter_id]["chart_test_id"],
+                    dom_id: "legend_" + plot_data[test_filter_id]["chart_test_id"],
+                    show: show,
+                    min: plot_data[test_filter_id]["min"],
+                    max: plot_data[test_filter_id]["max"],
+                    avg: (plot_data[test_filter_id]["sum"] / plot_data[test_filter_id]["data"].length).toFixed(2),
                 });
             }
 
@@ -848,6 +928,7 @@ $(document).ready(function () {
                 lines: lines_options,
                 points: points_options,
                 test_filter_id: test_filter_id,
+                chart_test_id: plot_data[test_filter_id]["chart_test_id"]
             });
         }
 
@@ -871,17 +952,22 @@ $(document).ready(function () {
         }
 
         // Add target goal dashed line to the plot.
-        if (this.chart_data["target_goal"]) {
+        if (this.chart_data["target_goal"] != null) {
+            if ($("#is_percentage_" + this.chart_id).prop("checked") == true) {
+                target_goal = parseFloat(this.chart_data["target_goal"]/y_max_pass).toFixed(4) * 100;
+            } else {
+                target_goal = this.chart_data["target_goal"];
+            }
+
 	    goal_data = [];
 
             if (this.chart_data.has_build_numbers) {
 	        for (var i in build_numbers) {
-	            goal_data.push([build_numbers[i],
-                                    this.chart_data["target_goal"]]);
+	            goal_data.push([build_numbers[i], target_goal]);
 	        }
             } else {
 	        for (key in dates) {
-	            goal_data.push([dates[key], this.chart_data["target_goal"]]);
+	            goal_data.push([dates[key], target_goal]);
 	        }
             }
 
@@ -908,8 +994,10 @@ $(document).ready(function () {
             $("#filter_links_container_" + this.chart_id).css("width", "80%");
         }
 
-        y_label = "Pass/Fail";
-        if (this.chart_data["chart_type"] != "pass/fail") {
+        y_label = "";
+        if (this.chart_data["chart_type"] == "measurement") {
+            y_label = "Pass/Fail";
+        } else if (this.chart_data["chart_type"] == "measurement") {
             if (this.chart_data.test_data[0]) {
                 y_label = this.chart_data.test_data[0].units;
             } else {
@@ -957,8 +1045,8 @@ $(document).ready(function () {
 	        container: "#legend_container_" + this.chart_id,
 	        labelFormatter: function(label, series) {
                     label_hidden = "<input type='hidden' " +
-                        "id='legend_" + series.test_filter_id + "' " +
-                        "value='" + series.test_filter_id + "'/>";
+                        "id='legend_" + series.chart_test_id + "' " +
+                        "value='" + series.chart_test_id + "'/>";
 		    return label + label_hidden;
 	        },
 	    },
@@ -970,6 +1058,12 @@ $(document).ready(function () {
                 axisLabelFontFamily: "Verdana",
             },
             canvas: true,
+            zoom: {
+                interactive: true
+            },
+            pan: {
+                interactive: true
+            },
         };
 
         // We cannot apply autoscaleMargin for y axis since y_max and y_min
@@ -988,9 +1082,89 @@ $(document).ready(function () {
 
         this.plot = $.plot($("#outer_container_" + this.chart_id + " #inner_container_" + this.chart_id), data, options);
 
-        // Setup click events and css in the legend.
-        this.setup_clickable();
-        this.setup_legend_css();
+        // Setup hooks, events, tooltips and css in the legend.
+        add_zoom_out(this);
+        setup_clickable(this);
+        setup_legend_tooltips(this);
+        setup_legend_css(this);
+
+        // Setup chart object reference.
+        this.plot.chart = this;
+        // Setup draw hook for plot.
+        this.plot.hooks.draw.push(function(plot, canvascontext) {
+            setup_clickable(plot.chart);
+            setup_legend_tooltips(plot.chart);
+            setup_legend_css(plot.chart);
+        });
+
+        this.plot.hooks.drawSeries.unshift(function(plot, canvascontext, series) {
+            reset_images(series);
+        });
+
+    }
+
+    reset_images = function(series) {
+        for (i = 0; i < series.data.length; i++) {
+            if (series.hasImage && series.hasImage[i]) {
+                series.hasImage[i] = null;
+            }
+        }
+    }
+
+    setup_legend_tooltips = function(chart) {
+        $("#legend_container_" + chart.chart_id + " td:last-child").each(function(index) {
+            $(this).attr("data-toggle", "tooltip");
+            $(this).attr("data-placement", "right");
+            var title =
+                "Min: " + chart.legend_items[index].min +
+                ", Max: " + chart.legend_items[index].max +
+                ", Avg: " + chart.legend_items[index].avg;
+            $(this).attr("title", title);
+        });
+    }
+
+    setup_clickable = function(chart) {
+
+        $("#legend_container_" + chart.chart_id + " table:first-child tbody tr").each(function(index) {
+
+            $(this).click(function(e) {
+
+                var show = chart.legend_items[index].show;
+                chart.legend_items[index].show = !show;
+
+                if (chart.chart_data["chart_type"] == "attributes") {
+                    // Chart id - attribute name combination.
+                    id_arr = chart.legend_items[index].chart_test_id.split("-");
+                    attr_array = [id_arr.shift(), id_arr.join("-")];
+                    chart.update_settings(attr_array[0], attr_array[1]);
+                } else {
+                    chart.update_settings(
+                        chart.legend_items[index].chart_test_id);
+                }
+                chart.update_plot();
+            });
+        });
+    }
+
+    setup_legend_css = function(chart) {
+
+        for (var i in chart.legend_items) {
+            if (chart.legend_items[i].show == true) {
+                $("#" + chart.legend_items[i].dom_id).parent().css("color", "#545454");
+            } else {
+                $("#" + chart.legend_items[i].dom_id).parent().css("color", "#999");
+            }
+        }
+    }
+
+    add_zoom_out = function(chart) {
+
+        $('<div class="zoom-out-button">zoom out</div>').click(
+            function () {
+                chart.plot.zoomOut();
+            }).appendTo("#inner_container_" + chart.chart_id).css(
+                "margin-left",
+                parseInt($("#inner_container_" + chart.chart_id).children().first().css("width")) - 90 + "px");
     }
 
     isNumeric = function(n) {
@@ -1055,20 +1229,98 @@ $(document).ready(function () {
     add_bug_link = function () {
         var current_bug = [];
 
-        $("#add-bug-dialog").bind('submit', function(e) {
-            var bug_url = $("#add-bug-dialog").find('input[name=bug_link]').val();
+        validate_submit = function() {
+            bug_url = $("#add-bug-dialog").find('input[name=bug_link]').val();
             if (!isValidUrl(bug_url)) {
-                e.preventDefault();
                 alert("'" + bug_url + "' is not a valid url!!");
+                return false;
             }
             if (current_bug.indexOf(bug_url) > -1) {
-                e.preventDefault();
                 alert("'" + bug_url + "' is already linked!!");
+                return false;
             }
-        });
+            return true;
+        }
+
 
         _submit = function () {
-            $(this).submit();
+
+            if (!validate_submit()) {
+                return;
+            }
+
+            var url = $("#add-bug-dialog").attr('url');
+            // Increase or decrease bug number
+            var increase = false;
+            if (url.indexOf("unlink") == -1) {
+                var increase = true;
+            }
+
+            bug_link = $("#add-bug-dialog").find('input[name=bug_link]').val();
+            data = {
+                csrfmiddlewaretoken: csrf_token,
+                bug_link: bug_link,
+                uuid: $("#add-bug-dialog").find('input[name=uuid]').val()
+            }
+
+            var relative_index = null;
+            if($("#add_bug_dialog").find('input[name=relative_index]')) {
+                relative_index = $("#add-bug-dialog").find('input[name=relative_index]').val();
+                data["relative_index"] = relative_index;
+            }
+
+            $.ajax({
+                url: url,
+                async: false,
+                type: "POST",
+                data: data,
+                beforeSend: function () {
+                    $('#loading_dialog').dialog('open');
+                },
+                success: function (data) {
+                    $('#loading_dialog').dialog('close');
+                    if (data[0].fields.analyzer_assigned_uuid) {
+                        uuid = data[0].fields.analyzer_assigned_uuid;
+                    } else {
+                        uuid = data[0].pk;
+                    }
+                    update_bug_dialog(uuid, increase, bug_link, relative_index);
+                    $("#add-bug-dialog").dialog('close');
+                },
+                error: function(data, status, error) {
+                    $('#loading_dialog').dialog('close');
+                    $("#add-bug-dialog").dialog('close');
+                    alert('Operation failed, please try again.');
+                }
+            });
+        }
+
+        update_bug_dialog = function (uuid, increase, bug_link, relative_index) {
+            // Find corresponding td field and change number of bugs in it.
+            if (relative_index) {
+                element = $("td[data-uuid='" + uuid + "'][data-relative_index='" + relative_index + "'] > .bug-link-container > .add-bug-link");
+            } else {
+                element = $("td[data-uuid='" + uuid + "'] > .bug-link-container > .add-bug-link");
+            }
+
+            bug_number = element.html().replace("[", "").replace("]", "");
+
+            if (increase) {
+                $("td[data-uuid='" + uuid + "'] > .bug-links").append(
+                    '<li class="bug-link">' + bug_link + '</li>'
+                );
+                bug_number ++;
+            } else {
+                bug_number --;
+
+                var links_element = $("td[data-uuid='" + uuid + "'] > .bug-links");
+                links_element.children().each(function() {
+                    if ($(this).html().indexOf(bug_link) != -1) {
+                        $(this).remove();
+                    }
+                });
+            }
+            element.html("[" + bug_number + "]");
         }
 
         var add_bug_dialog = $('#add-bug-dialog').dialog(
@@ -1135,20 +1387,18 @@ $(document).ready(function () {
                 var uuid = $(this).closest('td').data('uuid');
                 var rel_idx = $(this).closest('td').data('relative_index');
                 var chart_id = $(this).closest('td').data('chart-id');
-                var back_url = add_bug_dialog.find('input[name=back]').val().split('?')[0] + '?bug_links_chart_id=' + chart_id;
 
                 current_bug = get_linked_bugs($(this));
-                add_bug_dialog.find('input[name=back]').val(back_url);
                 add_bug_dialog.find('input[name=bug_link]').val('');
                 add_bug_dialog.find('input[name=uuid]').val(uuid);
 
                 if (rel_idx) {
                     add_bug_dialog.find('input[name=relative_index]').val(rel_idx);
-                    link_bug_url = testresult_link_bug_url
-                    unlink_bug_url = testresult_unlink_bug_url
+                    link_bug_url = testresult_link_bug_url;
+                    unlink_bug_url = testresult_unlink_bug_url;
                 } else {
-                    link_bug_url = testrun_link_bug_url
-                    unlink_bug_url = testrun_unlink_bug_url
+                    link_bug_url = testrun_link_bug_url;
+                    unlink_bug_url = testrun_unlink_bug_url;
                 }
 
                 if(current_bug.length) {
@@ -1175,9 +1425,9 @@ $(document).ready(function () {
                             if(confirm("Unlink '" + bug + "'")) {
                                 // unlink bug right now, so clear current_bug which is used for checking if the bug is duplicated when adding a bug
                                 current_bug = [];
-                                $('#add-bug-dialog').attr('action', unlink_bug_url);
+                                $('#add-bug-dialog').attr('url', unlink_bug_url);
                                 add_bug_dialog.find('input[name=bug_link]').val(bug);
-                                add_bug_dialog.submit();
+                                _submit();
                             }
                         }
                     );
@@ -1204,9 +1454,9 @@ $(document).ready(function () {
 
                             e.preventDefault();
                             if (confirm("Link '" + bug + "' to the '" + names.testrun + "' run of build" + names.buildnumber)) {
-                                $('#add-bug-dialog').attr('action', link_bug_url);
+                                $('#add-bug-dialog').attr('url', link_bug_url);
                                 add_bug_dialog.find('input[name=bug_link]').val(bug);
-                                add_bug_dialog.submit();
+                                _submit();
                             }
                         });
                 } else {
@@ -1215,22 +1465,38 @@ $(document).ready(function () {
 
                 var title = "Link a bug to the '" + names.testrun +
                     "' run of build " + names.buildnumber;
-                $('#add-bug-dialog').attr('action', link_bug_url);
+                $('#add-bug-dialog').attr('url', link_bug_url);
                 add_bug_dialog.dialog('option', 'title', title);
                 add_bug_dialog.dialog('open');
             }
         );
     }
 
+    init_loading_dialog = function() {
+    // Setup the loading image dialog.
+        $("#main_container").append('<div id="loading_dialog"></div>');
+        $("#loading_dialog").append('<img src="/static/dashboard_app/images/ajax-progress.gif" alt="Loading..." />');
+
+        $('#loading_dialog').dialog({
+            autoOpen: false,
+            title: '',
+            draggable: false,
+            height: 45,
+            width: 250,
+            modal: true,
+            resizable: false,
+            dialogClass: 'loading-dialog'
+        });
+
+        $('.loading-dialog div.ui-dialog-titlebar').hide();
+    }
+    init_loading_dialog();
+
     report = new ImageReport(chart_data);
     report.start();
 
     add_bug_link();
 
-    if (buglinks_chartid.length) {
-        $('#data_table_link_' + buglinks_chartid).click();
-    }
-
     $(window).resize(function () {
         report.redraw();
     });
