@@ -12,8 +12,8 @@ filters_callback = function(chart_id, filter_id, name) {
 
     $.ajax({
         url: url,
-        async: false,
         type: "POST",
+        async: false,
         data: {
             csrfmiddlewaretoken: csrf_token,
             chart_id: chart_id,
@@ -68,7 +68,7 @@ add_filter_container = function(data, filter_id, title) {
 
     content = '<hr><div class="filter-title">' + title + '</div>';
 
-    if ($('#id_chart_type').val() == "pass/fail") {
+    if ($('#id_chart_type').val() != "measurement") {
         test_label = "Tests";
     } else {
         test_label = "Test Cases";
@@ -85,9 +85,18 @@ add_filter_container = function(data, filter_id, title) {
     content += '<div class="selector"><div class="selector-available"><h2>' +
         'Select ' + test_label + '</h2>';
 
+
+    content += '<p id="tests_filter" class="selector-filter">' +
+        '<label for="tests_input">' +
+        '<img class="help-tooltip"' +
+        'src="/static/admin/img/selector-search.gif" alt=""></img>' +
+        '</label>' +
+        '<input id="tests_input" type="text"' +
+        'placeholder="Filter"></input></p>';
+
     content += '<select id="available_tests" multiple class="filtered">';
 
-    if ($('#id_chart_type').val() == "pass/fail") {
+    if ($('#id_chart_type').val() != "measurement") {
         content += generate_test_options(data, "pass/fail");
     }
 
@@ -122,7 +131,7 @@ generate_test_options = function(data, chart_type) {
 
     content = "";
     for (i in data) {
-        if (chart_type == "pass/fail") {
+        if (chart_type != "measurement") {
             content += '<option value="' + data[i].pk + '">' +
                 data[i].fields.test_id + '</option>';
         } else {
@@ -160,6 +169,20 @@ update_events = function(filter_id) {
     $('#test_select').change(function() {
        test_changed(filter_id, $(this).val());
     });
+
+    $('#tests_input').keyup(function() {
+       filter_available_tests($(this).val());
+    });
+}
+
+filter_available_tests = function(text) {
+    if (text != '') {
+        $('#available_tests option').filter(function() {
+            return $(this).text().toLowerCase().indexOf(text) != 0;
+        }).css("display", "none");
+    } else {
+        $('#available_tests option').css("display", "block");
+    }
 }
 
 move_options = function(from_element, to_element) {
@@ -172,7 +195,7 @@ add_selected_options = function() {
     // Adds options from chosen tests select box as hidden fields.
 
     $('#chosen_tests option').each(function() {
-        if ($('#id_chart_type').val() == "pass/fail") {
+        if ($('#id_chart_type').val() != "measurement") {
             field_name = "image_chart_tests";
         } else {
             field_name = "image_chart_test_cases";
@@ -191,7 +214,6 @@ test_changed = function(filter_id, test_id) {
 
         $.ajax({
             url: url,
-            async: false,
             data: {"id": filter_id, "test_id": test_id},
             beforeSend: function () {
                 $('#loading_dialog').dialog('open');
@@ -217,8 +239,8 @@ init_loading_dialog = function() {
         autoOpen: false,
         title: '',
         draggable: false,
-        height: 35,
-        width: 250,
+        height: 45,
+        width: 260,
         modal: true,
         resizable: false,
         dialogClass: 'loading-dialog'
@@ -249,7 +271,6 @@ open_test_edit = function(chart_filter_id, chart_test_id) {
 
     $.ajax({
         url: url,
-        async: false,
         data: {
             chart_filter_id: chart_filter_id,
             chart_test_id: chart_test_id},
