@@ -36,6 +36,9 @@ from lava_scheduler_daemon.jobsource import IJobSource
 import signal
 import platform
 
+MAX_RETRIES = 3
+
+
 try:
     from psycopg2 import InterfaceError, OperationalError
 except ImportError:
@@ -145,6 +148,16 @@ class DatabaseJobSource(object):
                 transaction.leave_transaction_management()
         return self.deferToThread(wrapper, *args, **kw)
 
+    def _commit_transaction(self, src=None):
+        for retry in range(MAX_RETRIES):
+            try:
+                transaction.commit()
+                self.logger.debug('%s transaction committed', src)
+                break
+            except TransactionRollbackError as err:
+                self.logger.warn('retrying transaction %s', err)
+                continue
+
     def _submit_health_check_jobs(self):
         """
         Checks which devices need a health check job and submits the needed
@@ -222,6 +235,9 @@ class DatabaseJobSource(object):
                         user=job.submitter)
                 device.current_job = job
                 device.state_transition_to(Device.RESERVED, message="Reserved for job %s" % job.display_id)
+                self._commit_transaction(src='%s state' % device.hostname)
+                self.logger.info('%s reserved for job %s', device.hostname,
+                                 job.id)
                 job.save()
                 device.save()
                 if device in devices:
@@ -237,11 +253,11 @@ class DatabaseJobSource(object):
         if os.path.exists(pidrecord):
             with open(pidrecord, 'r') as f:
                 pgid = int(f.read())
-                self.logger.info("Signalling SIGTERM to process group: %d" % pgid)
+                self.logger.info("Signalling SIGTERM to process group: %d", pgid)
                 try:
                     os.killpg(pgid, signal.SIGTERM)
                 except OSError as e:
-                    self.logger.info("Unable to kill process group %d: %s" % (pgid, e))
+                    self.logger.info("Unable to kill process group %d: %s", pgid, e)
                     os.unlink(pidrecord)
 
     def getJobList_impl(self):
@@ -263,7 +279,7 @@ class DatabaseJobSource(object):
 
         my_ready_jobs = filter(lambda job: job.is_ready_to_start, my_submitted_jobs)
 
-        transaction.commit()
+        self._commit_transaction(src='getJobList_impl')
         return my_ready_jobs
 
     def getJobList(self):
@@ -312,12 +328,15 @@ class DatabaseJobSource(object):
         if device.status == Device.RESERVED:
             msg = "Started running job %s" % job.display_id
             device.state_transition_to(Device.RUNNING, message=msg, job=job)
+            self._commit_transaction(src='%s state' % device.hostname)
+            self.logger.info('%s started running job %s', device.hostname,
+                             job.id)
         device.save()
         job.start_time = datetime.datetime.utcnow()
         shutil.rmtree(job.output_dir, ignore_errors=True)
         job.log_file.save('job-%s.log' % job.id, ContentFile(''), save=False)
         job.save()
-        transaction.commit()
+        self._commit_transaction(src='jobStarted_impl')
 
     def jobStarted(self, job):
         return self.deferForDB(self.jobStarted_impl, job)
@@ -328,7 +347,6 @@ class DatabaseJobSource(object):
         old_device_status = device.status
         new_device_status = None
         previous_state = device.previous_state()
-        MAX_RETRIES = 3
 
         if old_device_status == Device.RUNNING:
             new_device_status = previous_state
@@ -338,7 +356,7 @@ class DatabaseJobSource(object):
             new_device_status = previous_state
         else:
             self.logger.error(
-                "Unexpected device state in jobCompleted: %s" % device.status)
+                "Unexpected device state in jobCompleted: %s", device.status)
             new_device_status = Device.IDLE
         if new_device_status is None:
             new_device_status = Device.IDLE
@@ -351,7 +369,7 @@ class DatabaseJobSource(object):
                 if device.temporarydevice:
                     new_device_status = Device.RETIRED
             except TemporaryDevice.DoesNotExist:
-                self.logger.debug("%s is not a tmp device" % device.hostname)
+                self.logger.debug("%s is not a tmp device", device.hostname)
 
         device.device_version = _get_device_version(job.results_bundle)
         device.current_job = None
@@ -364,11 +382,13 @@ class DatabaseJobSource(object):
             job.status = TestJob.CANCELED
         else:
             self.logger.error(
-                "Unexpected job state in jobCompleted: %s" % job.status)
+                "Unexpected job state in jobCompleted: %s", job.status)
             job.status = TestJob.COMPLETE
 
         msg = "Job %s completed" % job.display_id
         device.state_transition_to(new_device_status, message=msg, job=job)
+        self._commit_transaction(src='%s state' % device.hostname)
+        self.logger.info('job %s completed on %s', job.id, device.hostname)
 
         if job.health_check:
             device.last_health_report_job = job
@@ -397,15 +417,7 @@ class DatabaseJobSource(object):
         job.submit_token = None
         device.save()
         job.save()
-        # notification needs to have the correct status in the database
-        for retry in range(MAX_RETRIES):
-            try:
-                transaction.commit()
-                self.logger.debug('%s job completed and status saved' % job.id)
-                break
-            except TransactionRollbackError as err:
-                self.logger.warn('Retrying %s job completion ... %s' % (job.id, err))
-                continue
+        self._commit_transaction(src='jobCompleted_impl')
         if utils.is_master():
             try:
                 job.send_summary_mails()
@@ -437,10 +449,10 @@ class DatabaseJobSource(object):
         #  may already have lost connection to the SchedulerMonitor via twisted.
         # Call TestJob.cancel to reset the TestJob status
         if len(cancel_list) > 0:
-            self.logger.debug("Number of jobs in cancelling status %d" % len(cancel_list))
+            self.logger.debug("Number of jobs in cancelling status %d", len(cancel_list))
             for job in cancel_list:
                 if job.actual_device and job.actual_device.hostname in self.my_devices():
-                    self.logger.debug("Looking for pid of dispatch job %s in %s" % (job.id, job.output_dir))
+                    self.logger.debug("Looking for pid of dispatch job %s in %s", job.id, job.output_dir)
                     self._kill_canceling(job)
                     device = Device.objects.get(hostname=job.actual_device.hostname)
                     if device.status == Device.RUNNING:
@@ -452,15 +464,16 @@ class DatabaseJobSource(object):
                                 if device.temporarydevice:
                                     previous_state = Device.RETIRED
                             except TemporaryDevice.DoesNotExist:
-                                self.logger.debug("%s is not a tmp device" % device.hostname)
-                        self.logger.debug("Transitioning %s to %s" % (device.hostname, previous_state))
+                                self.logger.debug("%s is not a tmp device", device.hostname)
+                        self.logger.debug("Transitioning %s to %s", device.hostname, previous_state)
                         device.current_job = None
                         msg = "Job %s cancelled" % job.display_id
                         device.state_transition_to(previous_state, message=msg,
                                                    job=job)
-                    self.logger.debug('Marking job %s as cancelled on %s' % (job.id, job.actual_device))
+                        self._commit_transaction(src='%s state' % device.hostname)
+                    self.logger.info('job %s cancelled on %s', job.id, job.actual_device)
                     job.cancel()
-                    transaction.commit()
+                    self._commit_transaction(src='_handle_cancelling_jobs')
 
 
 def _get_device_version(bundle):
