@@ -25,6 +25,7 @@
 # All comments below are strictly for development usage and
 # reference.
 
+import django
 
 # Administrator contact, used for sending
 # emergency email when something breaks
@@ -77,8 +78,6 @@ STATICFILES_MEDIA_DIRNAMES = (
     "static",
 )
 
-import django
-
 STATICFILES_PREPEND_LABEL_APPS = [
 ]
 
@@ -124,13 +123,18 @@ INSTALLED_APPS = [
     # Uncomment the next line to enable admin documentation:
     # 'django.contrib.admindocs',
     # Admin docs disabled due to: https://code.djangoproject.com/ticket/6681
-    'longerusername',
+    # 'longerusername',
     'linaro_django_xmlrpc',
     'lava_markitup',  # Support app for MarkItUp in LAVA
-    'south',
     'google_analytics',
 ]
 
+if django.VERSION < (1, 7):
+    # Django 1.7 has built-in migration suppport
+    INSTALLED_APPS += ['south']
+else:
+    TEST_RUNNER = 'django.test.runner.DiscoverRunner'
+
 try:
     import devserver
     INSTALLED_APPS += ['devserver']
@@ -146,6 +150,7 @@ TEMPLATE_CONTEXT_PROCESSORS = [
     "django.core.context_processors.static",
     "lava_server.context_processors.lava",
     "lava_server.context_processors.openid_available",
+    "lava_server.context_processors.ldap_available",
 ]
 
 
@@ -153,10 +158,29 @@ AUTHENTICATION_BACKENDS = (
     # Uncomment CrowdRestBackend and comment OpenIDBackend to enable
     # Atlassian Crowd auth.
     # 'crowdrest.backend.CrowdRestBackend',
+
+    # Uncomment LDAPBackend and comment OpenIDBackend to enable LDAP auth and
+    # make sure LDAP auth parameters are set properly.
+    # 'django_auth_ldap.backend.LDAPBackend',
+
     'django_openid_auth.auth.OpenIDBackend',
     'django.contrib.auth.backends.ModelBackend',
 )
 
+# Uncomment the following lines to configure LDAP auth parameters. For details
+# on configuration parameters,
+# see: https://pythonhosted.org/django-auth-ldap/authentication.html
+# AUTH_LDAP_SERVER_URI = "ldap://yourldapservername"
+# AUTH_LDAP_BIND_DN = ""
+# AUTH_LDAP_BIND_PASSWORD = ""
+# AUTH_LDAP_USER_DN_TEMPLATE = "uid=%(user)s,ou=users,dc=example,dc=com"
+
+# Uncomment the following lines, if you need logging enabled for LDAP auth.
+# import logging
+# logger = logging.getLogger('django_auth_ldap')
+# logger.addHandler(logging.StreamHandler())
+# logger.setLevel(logging.DEBUG)
+
 OPENID_CREATE_USERS = True
 OPENID_LAUNCHPAD_TEAMS_MAPPING_AUTO = False
 OPENID_UPDATE_DETAILS_FROM_SREG = True
