@@ -70,7 +70,7 @@ DEBUG_PROPAGATE_EXCEPTIONS = True
 TEMPLATE_DEBUG = DEBUG
 
 # It would be good to have rails-like configuration file in the future
-devel_db = os.getenv("DEVEL_DB", "sqlite")
+devel_db = os.getenv("DEVEL_DB", "pgsql")
 if devel_db == "pgsql":
     DATABASES = {
         'default': {
@@ -80,15 +80,6 @@ if devel_db == "pgsql":
             'PASSWORD': 'devel',
             'HOST': 'localhost',
             'PORT': ''}}
-elif devel_db == "sqlite":
-    DATABASES = {
-        'default': {
-            'ENGINE': 'django.db.backends.sqlite3',
-            'NAME': os.path.join(PROJECT_STATE_DIR, 'development.db'),
-            'USER': '',
-            'PASSWORD': '',
-            'HOST': '',
-            'PORT': ''}}
 elif devel_db == "nosql":
     raise ValueError("not yet ;-)")
 else:
@@ -99,6 +90,9 @@ else:
 # Example: "/home/media/media.lawrence.com/"
 MEDIA_ROOT = os.path.join(PROJECT_STATE_DIR, "media", devel_db)
 
+# Absolute filesystem path to the directory that will hold archived files.
+ARCHIVE_ROOT = os.path.join(PROJECT_STATE_DIR, "archive", devel_db)
+
 # Absolute filesystem path to the directory that will hold static, read only
 # files collected from all applications.
 # Example: "/home/media/static.lawrence.com/"
